import sys
import os
from pathlib import Path
import subprocess as sp
import time
import shutil

# Add this new section
from pkg_resources import resource_filename
from .Scripts import essential_annotation
from .Scripts import draw_plasmid



def process_plasmid(plasmid, pathofdir, default_db_path,oric_data,orit_data,plasmidfinder_data,transposon_data):
    start_time = time.time()
    print(plasmid)
    fasta_file = os.path.join(pathofdir, f"{plasmid}.fasta")
    os.system(f'prodigal -i "{fasta_file}" -o tmp_files/{plasmid}prodigal.gbk -f gbk > /dev/null 2>&1')
    length_of_plasmid_sequence = essential_annotation.getsequencelength(plasmid, pathofdir)
    positions_of_coding_sequences = essential_annotation.getpositionsofCDS(plasmid)
    complement_start, complement_end = essential_annotation.complementpositions(plasmid)
    DNA_Sequence_of_fasta = essential_annotation.getthesequences(plasmid, pathofdir)
    DNA_cds_list = essential_annotation.getlistofDNACDS(positions_of_coding_sequences, DNA_Sequence_of_fasta)
    essential_annotation.makequeryfastafordbsearch(DNA_cds_list)
    database = essential_annotation.makedatabasefromcsvfile(default_db_path)
    Initial_dataframe = essential_annotation.initial_blast_against_database(DNA_cds_list, positions_of_coding_sequences, database)
    oric_dataframe = essential_annotation.blast_against_oric_dataframe(oric_data, plasmid, pathofdir)
    orit_dataframe = essential_annotation.blast_against_orit_dataframe(orit_data, plasmid, pathofdir)
    replicon_dataframe = essential_annotation.blast_against_replicon_database(plasmidfinder_data, plasmid, pathofdir)
    transposon_dataframe = essential_annotation.blast_against_transposon_database(transposon_data, plasmid, pathofdir, Initial_dataframe)
    final_dataframe = essential_annotation.process_final_dataframe(Initial_dataframe, replicon_dataframe, oric_dataframe, orit_dataframe, transposon_dataframe)
    output_directory = f"Output_directory/{plasmid}/"
    if not os.path.exists(output_directory):
        os.makedirs(output_directory)
    final_dataframe.to_csv(output_directory + f"Annotation_table_for_{plasmid}.csv")
    output_path_for_genbank = output_directory + f"Annotation_gbk_file_for_{plasmid}.gbk"
    essential_annotation.make_genbank_file(DNA_Sequence_of_fasta, final_dataframe, output_path_for_genbank, plasmid)
    plasmid_map_path = output_directory + f"Annotated_Map_for_{plasmid}.png"
    draw_plasmid.draw_plasmid_map_from_genbank_file(output_path_for_genbank, plasmid_map_path)
    end_time = time.time()
    duration = end_time - start_time
    print(f"The function took {duration} seconds to complete for {plasmid}.")
    


def main():
    if not os.path.exists("tmp_files"):
        os.makedirs("tmp_files")

    # Get the absolute path of the script's directory
    script_dir = os.path.dirname(os.path.abspath(__file__))

    # Set default paths relative to the script's directory
    default_db_path = os.path.join(script_dir, "Databases/DatabaseV5.1.3.csv")
    oric_data = os.path.join(script_dir, "Databases/oric.fna")
    orit_data = os.path.join(script_dir, "Databases/orit.fna")
    transposon_data = os.path.join(script_dir, "Databases/transposon.fasta")
    plasmidfinder_data = os.path.join(script_dir, "Databases/plasmidfinder.fasta")

    default_path = os.path.join(script_dir, "fasta_files/")

    if len(sys.argv) > 1:
        input_path = os.path.abspath(sys.argv[1])
    else:
        input_path = default_path

    if os.path.isdir(input_path):
        entries = os.listdir(input_path)
        plasmid_list = [os.path.splitext(file)[0] for file in entries if os.path.isfile(os.path.join(input_path, file)) and file.endswith('.fasta')]
        for plasmid in plasmid_list:
            process_plasmid(plasmid, input_path, default_db_path, oric_data, orit_data, plasmidfinder_data, transposon_data)
    elif os.path.isfile(input_path) and input_path.endswith('.fasta'):
        plasmid = os.path.splitext(os.path.basename(input_path))[0]
        process_plasmid(plasmid, os.path.dirname(input_path), default_db_path, oric_data, orit_data, plasmidfinder_data, transposon_data)
    else:
        print("Invalid path or file type. Please provide a valid directory or FASTA file.")

    shutil.rmtree('tmp_files', ignore_errors=True)
    shutil.rmtree('makedb_folder', ignore_errors=True)

if __name__ == "__main__":
    main()
